# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'simsapa/assets/ui/document_reader_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DocumentReaderWindow(object):
    def setupUi(self, DocumentReaderWindow):
        DocumentReaderWindow.setObjectName("DocumentReaderWindow")
        DocumentReaderWindow.resize(856, 581)
        DocumentReaderWindow.setBaseSize(QtCore.QSize(800, 600))
        self.central_widget = QtWidgets.QWidget(DocumentReaderWindow)
        self.central_widget.setObjectName("central_widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.central_widget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.main_layout = QtWidgets.QVBoxLayout()
        self.main_layout.setObjectName("main_layout")
        self.splitter = QtWidgets.QSplitter(self.central_widget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.sidebar_layout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.sidebar_layout.setContentsMargins(0, 0, 0, 0)
        self.sidebar_layout.setObjectName("sidebar_layout")
        self.sidebar_tabs = QtWidgets.QTabWidget(self.verticalLayoutWidget_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sidebar_tabs.sizePolicy().hasHeightForWidth())
        self.sidebar_tabs.setSizePolicy(sizePolicy)
        self.sidebar_tabs.setObjectName("sidebar_tabs")
        self.thumbs_tab = QtWidgets.QWidget()
        self.thumbs_tab.setObjectName("thumbs_tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.thumbs_tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.thumbs_view = QtWidgets.QWidget(self.thumbs_tab)
        self.thumbs_view.setObjectName("thumbs_view")
        self.verticalLayout_2.addWidget(self.thumbs_view)
        self.sidebar_tabs.addTab(self.thumbs_tab, "")
        self.annotations_tab = QtWidgets.QWidget()
        self.annotations_tab.setObjectName("annotations_tab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.annotations_tab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.annotations_view = QtWidgets.QWidget(self.annotations_tab)
        self.annotations_view.setObjectName("annotations_view")
        self.verticalLayout_3.addWidget(self.annotations_view)
        self.sidebar_tabs.addTab(self.annotations_tab, "")
        self.bookmarks_tab = QtWidgets.QWidget()
        self.bookmarks_tab.setObjectName("bookmarks_tab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.bookmarks_tab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.bookmarks_list = QtWidgets.QListWidget(self.bookmarks_tab)
        self.bookmarks_list.setObjectName("bookmarks_list")
        self.verticalLayout_4.addWidget(self.bookmarks_list)
        self.sidebar_tabs.addTab(self.bookmarks_tab, "")
        self.contents_tab = QtWidgets.QWidget()
        self.contents_tab.setObjectName("contents_tab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.contents_tab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.contents_list = QtWidgets.QListView(self.contents_tab)
        self.contents_list.setObjectName("contents_list")
        self.verticalLayout_5.addWidget(self.contents_list)
        self.sidebar_tabs.addTab(self.contents_tab, "")
        self.sidebar_layout.addWidget(self.sidebar_tabs)
        self.verticalLayoutWidget_3 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_3.setObjectName("verticalLayoutWidget_3")
        self.content_layout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_3)
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_layout.setObjectName("content_layout")
        self.page_number_layout = QtWidgets.QHBoxLayout()
        self.page_number_layout.setObjectName("page_number_layout")
        self.label = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setText("")
        self.label.setObjectName("label")
        self.page_number_layout.addWidget(self.label)
        self.current_page_input = QtWidgets.QSpinBox(self.verticalLayoutWidget_3)
        self.current_page_input.setMinimumSize(QtCore.QSize(80, 0))
        self.current_page_input.setMaximumSize(QtCore.QSize(80, 16777215))
        self.current_page_input.setAlignment(QtCore.Qt.AlignCenter)
        self.current_page_input.setMinimum(1)
        self.current_page_input.setMaximum(999999)
        self.current_page_input.setObjectName("current_page_input")
        self.page_number_layout.addWidget(self.current_page_input)
        self.page_current_of_total = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        self.page_current_of_total.setMinimumSize(QtCore.QSize(80, 0))
        self.page_current_of_total.setAlignment(QtCore.Qt.AlignCenter)
        self.page_current_of_total.setObjectName("page_current_of_total")
        self.page_number_layout.addWidget(self.page_current_of_total)
        self.label_4 = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setText("")
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.page_number_layout.addWidget(self.label_4)
        self.content_layout.addLayout(self.page_number_layout)
        self.scrollArea = QtWidgets.QScrollArea(self.verticalLayoutWidget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setMinimumSize(QtCore.QSize(400, 0))
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 398, 444))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.content_page = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.content_page.sizePolicy().hasHeightForWidth())
        self.content_page.setSizePolicy(sizePolicy)
        self.content_page.setText("")
        self.content_page.setObjectName("content_page")
        self.verticalLayout.addWidget(self.content_page)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.content_layout.addWidget(self.scrollArea)
        self.verticalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.notes_layout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.notes_layout.setContentsMargins(0, 0, 0, 0)
        self.notes_layout.setObjectName("notes_layout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.add_note_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.add_note_button.setObjectName("add_note_button")
        self.horizontalLayout.addWidget(self.add_note_button)
        self.remove_note_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.remove_note_button.setObjectName("remove_note_button")
        self.horizontalLayout.addWidget(self.remove_note_button)
        self.notes_layout.addLayout(self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_2.setObjectName("label_2")
        self.notes_layout.addWidget(self.label_2)
        self.front_input = QtWidgets.QPlainTextEdit(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.front_input.sizePolicy().hasHeightForWidth())
        self.front_input.setSizePolicy(sizePolicy)
        self.front_input.setMaximumSize(QtCore.QSize(16777215, 50))
        self.front_input.setObjectName("front_input")
        self.notes_layout.addWidget(self.front_input)
        self.label_5 = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_5.setObjectName("label_5")
        self.notes_layout.addWidget(self.label_5)
        self.back_input = QtWidgets.QPlainTextEdit(self.verticalLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.back_input.sizePolicy().hasHeightForWidth())
        self.back_input.setSizePolicy(sizePolicy)
        self.back_input.setMaximumSize(QtCore.QSize(16777215, 50))
        self.back_input.setObjectName("back_input")
        self.notes_layout.addWidget(self.back_input)
        self.label_3 = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_3.setObjectName("label_3")
        self.notes_layout.addWidget(self.label_3)
        self.notes_list = QtWidgets.QListView(self.verticalLayoutWidget)
        self.notes_list.setObjectName("notes_list")
        self.notes_layout.addWidget(self.notes_list)
        self.main_layout.addWidget(self.splitter)
        self.horizontalLayout_2.addLayout(self.main_layout)
        DocumentReaderWindow.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(DocumentReaderWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 856, 20))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Edit = QtWidgets.QMenu(self.menubar)
        self.menu_Edit.setObjectName("menu_Edit")
        self.menu_Windows = QtWidgets.QMenu(self.menubar)
        self.menu_Windows.setObjectName("menu_Windows")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        self.menuGo = QtWidgets.QMenu(self.menubar)
        self.menuGo.setObjectName("menuGo")
        DocumentReaderWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(DocumentReaderWindow)
        self.statusbar.setObjectName("statusbar")
        DocumentReaderWindow.setStatusBar(self.statusbar)
        self.windows_toolbar = QtWidgets.QToolBar(DocumentReaderWindow)
        self.windows_toolbar.setObjectName("windows_toolbar")
        DocumentReaderWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.windows_toolbar)
        self.doc_toolbar = QtWidgets.QToolBar(DocumentReaderWindow)
        self.doc_toolbar.setObjectName("doc_toolbar")
        DocumentReaderWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.doc_toolbar)
        self.action_Copy = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Copy.setObjectName("action_Copy")
        self.action_Paste = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Paste.setObjectName("action_Paste")
        self.action_Quit = QtWidgets.QAction(DocumentReaderWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/close"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Quit.setIcon(icon)
        self.action_Quit.setObjectName("action_Quit")
        self.action_Sutta_Search = QtWidgets.QAction(DocumentReaderWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/book"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Sutta_Search.setIcon(icon1)
        self.action_Sutta_Search.setObjectName("action_Sutta_Search")
        self.action_Dictionary_Search = QtWidgets.QAction(DocumentReaderWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/dictionary"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Dictionary_Search.setIcon(icon2)
        self.action_Dictionary_Search.setObjectName("action_Dictionary_Search")
        self.action_About = QtWidgets.QAction(DocumentReaderWindow)
        self.action_About.setObjectName("action_About")
        self.action_Website = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Website.setObjectName("action_Website")
        self.action_Close_Window = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Close_Window.setObjectName("action_Close_Window")
        self.action_Document_Reader = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Document_Reader.setObjectName("action_Document_Reader")
        self.action_Open = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Open.setObjectName("action_Open")
        self.action_Next_Page = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Next_Page.setObjectName("action_Next_Page")
        self.action_Previous_Page = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Previous_Page.setObjectName("action_Previous_Page")
        self.action_Go_to_Page = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Go_to_Page.setObjectName("action_Go_to_Page")
        self.action_Beginning = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Beginning.setObjectName("action_Beginning")
        self.action_End = QtWidgets.QAction(DocumentReaderWindow)
        self.action_End.setObjectName("action_End")
        self.action_Library = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Library.setObjectName("action_Library")
        self.action_Notes = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Notes.setObjectName("action_Notes")
        self.action_Dictionaries_Manager = QtWidgets.QAction(DocumentReaderWindow)
        self.action_Dictionaries_Manager.setObjectName("action_Dictionaries_Manager")
        self.menu_File.addAction(self.action_Open)
        self.menu_File.addAction(self.action_Close_Window)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.action_Quit)
        self.menu_Edit.addAction(self.action_Copy)
        self.menu_Edit.addAction(self.action_Paste)
        self.menu_Windows.addAction(self.action_Sutta_Search)
        self.menu_Windows.addAction(self.action_Dictionary_Search)
        self.menu_Windows.addAction(self.action_Dictionaries_Manager)
        self.menu_Windows.addAction(self.action_Document_Reader)
        self.menu_Windows.addAction(self.action_Library)
        self.menu_Windows.addAction(self.action_Notes)
        self.menu_Help.addAction(self.action_Website)
        self.menu_Help.addAction(self.action_About)
        self.menuGo.addAction(self.action_Previous_Page)
        self.menuGo.addAction(self.action_Next_Page)
        self.menuGo.addSeparator()
        self.menuGo.addAction(self.action_Beginning)
        self.menuGo.addAction(self.action_End)
        self.menuGo.addSeparator()
        self.menuGo.addAction(self.action_Go_to_Page)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Edit.menuAction())
        self.menubar.addAction(self.menu_Windows.menuAction())
        self.menubar.addAction(self.menuGo.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.windows_toolbar.addAction(self.action_Sutta_Search)
        self.windows_toolbar.addAction(self.action_Dictionary_Search)
        self.doc_toolbar.addAction(self.action_Beginning)
        self.doc_toolbar.addAction(self.action_Previous_Page)
        self.doc_toolbar.addAction(self.action_Go_to_Page)
        self.doc_toolbar.addAction(self.action_Next_Page)
        self.doc_toolbar.addAction(self.action_End)

        self.retranslateUi(DocumentReaderWindow)
        self.sidebar_tabs.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DocumentReaderWindow)

    def retranslateUi(self, DocumentReaderWindow):
        _translate = QtCore.QCoreApplication.translate
        DocumentReaderWindow.setWindowTitle(_translate("DocumentReaderWindow", "Document Reader - Simsapa"))
        self.sidebar_tabs.setTabText(self.sidebar_tabs.indexOf(self.thumbs_tab), _translate("DocumentReaderWindow", "Thumbnails"))
        self.sidebar_tabs.setTabText(self.sidebar_tabs.indexOf(self.annotations_tab), _translate("DocumentReaderWindow", "Annotations"))
        self.sidebar_tabs.setTabText(self.sidebar_tabs.indexOf(self.bookmarks_tab), _translate("DocumentReaderWindow", "Bookmarks"))
        self.sidebar_tabs.setTabText(self.sidebar_tabs.indexOf(self.contents_tab), _translate("DocumentReaderWindow", "Contents"))
        self.page_current_of_total.setText(_translate("DocumentReaderWindow", "_ of _"))
        self.add_note_button.setText(_translate("DocumentReaderWindow", "Add"))
        self.remove_note_button.setText(_translate("DocumentReaderWindow", "Remove"))
        self.label_2.setText(_translate("DocumentReaderWindow", "Front"))
        self.label_5.setText(_translate("DocumentReaderWindow", "Back"))
        self.label_3.setText(_translate("DocumentReaderWindow", "Notes for this page"))
        self.menu_File.setTitle(_translate("DocumentReaderWindow", "&File"))
        self.menu_Edit.setTitle(_translate("DocumentReaderWindow", "&Edit"))
        self.menu_Windows.setTitle(_translate("DocumentReaderWindow", "&Windows"))
        self.menu_Help.setTitle(_translate("DocumentReaderWindow", "&Help"))
        self.menuGo.setTitle(_translate("DocumentReaderWindow", "Go"))
        self.windows_toolbar.setWindowTitle(_translate("DocumentReaderWindow", "toolBar"))
        self.doc_toolbar.setWindowTitle(_translate("DocumentReaderWindow", "toolBar"))
        self.action_Copy.setText(_translate("DocumentReaderWindow", "&Copy"))
        self.action_Paste.setText(_translate("DocumentReaderWindow", "&Paste"))
        self.action_Quit.setText(_translate("DocumentReaderWindow", "&Quit"))
        self.action_Quit.setShortcut(_translate("DocumentReaderWindow", "Ctrl+Q"))
        self.action_Sutta_Search.setText(_translate("DocumentReaderWindow", "&Sutta Search"))
        self.action_Sutta_Search.setShortcut(_translate("DocumentReaderWindow", "F5"))
        self.action_Dictionary_Search.setText(_translate("DocumentReaderWindow", "&Dictionary Search"))
        self.action_Dictionary_Search.setShortcut(_translate("DocumentReaderWindow", "F6"))
        self.action_About.setText(_translate("DocumentReaderWindow", "&About"))
        self.action_Website.setText(_translate("DocumentReaderWindow", "&Website"))
        self.action_Close_Window.setText(_translate("DocumentReaderWindow", "&Close Window"))
        self.action_Document_Reader.setText(_translate("DocumentReaderWindow", "D&ocument Reader"))
        self.action_Document_Reader.setToolTip(_translate("DocumentReaderWindow", "Document Reader"))
        self.action_Document_Reader.setShortcut(_translate("DocumentReaderWindow", "F7"))
        self.action_Open.setText(_translate("DocumentReaderWindow", "&Open..."))
        self.action_Open.setShortcut(_translate("DocumentReaderWindow", "Ctrl+O"))
        self.action_Next_Page.setText(_translate("DocumentReaderWindow", "&Next Page"))
        self.action_Next_Page.setShortcut(_translate("DocumentReaderWindow", "PgDown"))
        self.action_Previous_Page.setText(_translate("DocumentReaderWindow", "&Previous Page"))
        self.action_Previous_Page.setShortcut(_translate("DocumentReaderWindow", "PgUp"))
        self.action_Go_to_Page.setText(_translate("DocumentReaderWindow", "&Go to Page..."))
        self.action_Go_to_Page.setShortcut(_translate("DocumentReaderWindow", "Ctrl+G"))
        self.action_Beginning.setText(_translate("DocumentReaderWindow", "&Beginning"))
        self.action_Beginning.setShortcut(_translate("DocumentReaderWindow", "Ctrl+Home"))
        self.action_End.setText(_translate("DocumentReaderWindow", "&End"))
        self.action_End.setShortcut(_translate("DocumentReaderWindow", "Ctrl+End"))
        self.action_Library.setText(_translate("DocumentReaderWindow", "&Library"))
        self.action_Library.setShortcut(_translate("DocumentReaderWindow", "F8"))
        self.action_Notes.setText(_translate("DocumentReaderWindow", "&Notes"))
        self.action_Notes.setToolTip(_translate("DocumentReaderWindow", "Notes"))
        self.action_Notes.setShortcut(_translate("DocumentReaderWindow", "F9"))
        self.action_Dictionaries_Manager.setText(_translate("DocumentReaderWindow", "Dictionaries &Manager"))
        self.action_Dictionaries_Manager.setShortcut(_translate("DocumentReaderWindow", "F10"))
from simsapa.assets import icons_rc



from pathlib import Path
import requests


def download_file(url: str, folder_path: Path) -> Path:
    file_name = url.split('/')[-1]
    file_path = folder_path.joinpath(file_name)

    with requests.get(url, stream=True) as r:
        r.raise_for_status()
        with open(file_path, 'wb') as f:
            for chunk in r.iter_content(chunk_size=8192):
                f.write(chunk)

    return file_path



import os
from functools import partial
import tarfile
import logging as _logging

from PyQt5.QtCore import Qt  # type: ignore
from PyQt5.QtCore import QObject, QThread, pyqtSignal
from PyQt5.QtWidgets import (QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QMainWindow)  # type: ignore
from PyQt5.QtGui import QMovie  # type: ignore

from ..app.types import ASSETS_DIR, APP_DB_PATH
from ..app.helpers import download_file

from simsapa.assets import icons_rc  # noqa: F401

logger = _logging.getLogger(__name__)


class DownloadAppdataWindow(QMainWindow):
    def __init__(self) -> None:
        super().__init__()
        self.setWindowTitle("Download Application Assets")
        self.setFixedSize(300, 300)
        self.setWindowFlags(Qt.WindowStaysOnTopHint)

        self._central_widget = QWidget(self)
        self.setCentralWidget(self._central_widget)

        self._layout = QVBoxLayout()
        self._central_widget.setLayout(self._layout)

        self._msg = QLabel("The application database\nwas not found on this system.\n\nDownload it now?")
        self._msg.setAlignment(Qt.AlignCenter)

        self._layout.addWidget(self._msg)

        self._animation = QLabel(self)
        self._animation.setAlignment(Qt.AlignCenter)
        self._layout.addWidget(self._animation)

        buttons_layout = QHBoxLayout()
        buttons_layout.setContentsMargins(0, 20, 0, 10)

        self._download_button = QPushButton("Download")
        self._download_button.setFixedSize(100, 30)
        self._quit_button = QPushButton("Quit")
        self._quit_button.setFixedSize(100, 30)

        self._download_button.clicked.connect(partial(self._run_download))
        self._quit_button.clicked.connect(partial(self.close))

        buttons_layout.addWidget(self._quit_button)
        buttons_layout.addWidget(self._download_button)

        self._layout.addLayout(buttons_layout)

    def startAnimation(self):
        self._movie.start()

    def stopAnimation(self):
        self._movie.stop()

    def _run_download(self):
        self.thread = QThread()
        self.worker = DownloadWorker()
        self.worker.moveToThread(self.thread)

        self.thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.thread.quit)
        self.worker.finished.connect(self.worker.deleteLater)
        self.thread.finished.connect(self.thread.deleteLater)

        self.thread.start()

        self._msg.setText("Downloading ...")
        self._download_button.setEnabled(False)

        self._movie = QMovie(':simsapa-loading')
        self._animation.setMovie(self._movie)

        self.startAnimation()

        self.thread.finished.connect(self._download_finished)

    def _download_finished(self):
        self.stopAnimation()
        self._animation.deleteLater()

        self._msg.setText("Download completed.\n\nQuit and start the application again.")


class DownloadWorker(QObject):
    finished = pyqtSignal()

    def run(self):
        download_extract_appdata()
        self.finished.emit()


def download_extract_appdata() -> bool:
    tar_file_path = download_file(
        'https://ssp.a-buddha-ujja.hu/appdata/appdata.tar.bz2',
        ASSETS_DIR
    )

    tar = tarfile.open(tar_file_path, "r:bz2")
    tar.extractall(ASSETS_DIR)
    tar.close()

    os.remove(tar_file_path)

    if not APP_DB_PATH.exists():
        logger.error(f"File not found: {APP_DB_PATH}")



from functools import partial
from typing import List

from PyQt5.QtWidgets import (QApplication, QMainWindow, QFileDialog)  # type: ignore

from .types import AppData  # type: ignore

from ..layouts.sutta_search import SuttaSearchWindow, SuttaSearchCtrl  # type: ignore
from ..layouts.dictionary_search import DictionarySearchWindow, DictionarySearchCtrl  # type: ignore
from ..layouts.dictionaries_manager import DictionariesManagerWindow  # type: ignore
from ..layouts.document_reader import DocumentReaderWindow, DocumentReaderCtrl  # type: ignore
from ..layouts.library_browser import LibraryBrowserWindow  # type: ignore
from ..layouts.notes_browser import NotesBrowserWindow  # type: ignore


class AppWindows:
    def __init__(self, app: QApplication, app_data: AppData):
        self._app = app
        self._app_data = app_data
        self._windows: List[QMainWindow] = []

    def _new_sutta_search_window(self):
        view = SuttaSearchWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        SuttaSearchCtrl(view)
        self._windows.append(view)

    def _new_dictionary_search_window(self):
        view = DictionarySearchWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        DictionarySearchCtrl(view)
        self._windows.append(view)

    def _new_dictionaries_manager_window(self):
        view = DictionariesManagerWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        self._windows.append(view)

    def _new_library_browser_window(self):
        view = LibraryBrowserWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        self._windows.append(view)

    def _new_notes_browser_window(self):
        view = NotesBrowserWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        self._windows.append(view)

    def _new_document_reader_window(self, file_path=None):
        view = DocumentReaderWindow(self._app_data)
        self._connect_signals(view)
        view.show()
        DocumentReaderCtrl(view)

        if file_path is not None and file_path is not False and len(file_path) > 0:
            view.open_doc(file_path)

        self._windows.append(view)

    def _open_selected_document(self, view: QMainWindow):
        doc = view.get_selected_document()
        if doc:
            self._new_document_reader_window(doc.filepath)

    def _open_file_dialog(self, view: QMainWindow):
        try:
            view.open_file_dialog()
        except AttributeError:
            file_path, _ = QFileDialog.getOpenFileName(
                None,
                "Open File...",
                "",
                "PDF or Epub Files (*.pdf *.epub)")

            if len(file_path) != 0:
                self._new_document_reader_window(file_path)

    def _close_all_windows(self):
        for w in self._windows:
            w.close()

    def _quit_app(self):
        self._close_all_windows()
        self._app.quit()

    def _connect_signals(self, view: QMainWindow):
        view.action_Open \
            .triggered.connect(partial(self._open_file_dialog, view))

        try:
            view.action_Open_Selected \
                .triggered.connect(partial(self._open_selected_document, view))
        except Exception:
            pass

        view.action_Quit \
            .triggered.connect(partial(self._quit_app))
        view.action_Sutta_Search \
            .triggered.connect(partial(self._new_sutta_search_window))
        view.action_Dictionary_Search \
            .triggered.connect(partial(self._new_dictionary_search_window))
        view.action_Dictionaries_Manager \
            .triggered.connect(partial(self._new_dictionaries_manager_window))
        view.action_Document_Reader \
            .triggered.connect(partial(self._new_document_reader_window))
        view.action_Library \
            .triggered.connect(partial(self._new_library_browser_window))
        view.action_Notes \
            .triggered.connect(partial(self._new_notes_browser_window))



from pathlib import Path
import os
import os.path
import appdirs  # type: ignore
import logging as _logging

from sqlalchemy import create_engine, text  # type: ignore
from sqlalchemy_utils import database_exists, create_database  # type: ignore
from sqlalchemy.orm import sessionmaker  # type: ignore

import simsapa.app.db_models as db_models

logger = _logging.getLogger(__name__)

SIMSAPA_DIR = Path(appdirs.user_data_dir('simsapa'))
ASSETS_DIR = SIMSAPA_DIR.joinpath('assets')
APP_DB_PATH = ASSETS_DIR.joinpath('appdata.sqlite3')
USER_DB_PATH = ASSETS_DIR.joinpath('userdata.sqlite3')


class AppData:
    def __init__(self, app_db_path=None, user_db_path=None):

        if app_db_path is None:
            app_db_path = self._find_app_data_or_exit()

        if user_db_path is None:
            user_db_path = self._find_user_data_or_create()

        self.app_db_conn, self.app_db_session = self._connect_to_db(app_db_path)
        self.user_db_conn, self.user_db_session = self._connect_to_db(user_db_path)

    def _connect_to_db(self, db_path):
        if os.path.isfile(db_path):
            try:
                engine = create_engine(f"sqlite+pysqlite:///{db_path}", echo=False, future=True)
                db_conn = engine.connect()
                Session = sessionmaker(engine)
                Session.configure(bind=engine)
                db_session = Session()
            except Exception as e:
                logger.error(f"Can't connect to database: {db_path}")
                print(e)
                exit(1)
        else:
            logger.error(f"Database file doesn't exist: {db_path}")
            exit(1)

        return (db_conn, db_session)

    def _find_app_data_or_exit(self):
        if not APP_DB_PATH.exists():
            logger.error("Cannot find appdata.sqlite3")
            exit(1)
        else:
            return APP_DB_PATH

    def _find_user_data_or_create(self):
        if not USER_DB_PATH.exists():
            logger.info("Cannot find userdata.sqlite3, creating it")

            engine = create_engine(f"sqlite+pysqlite:///{USER_DB_PATH}",
                                   echo=False, future=True)
            if not database_exists(engine.url):
                create_database(engine.url)

            with engine.connect() as db_conn:
                for s in db_models.USERDATA_CREATE_SCHEMA_SQL.split(';'):
                    db_conn.execute(text(s))

        return USER_DB_PATH


class DictWord:
    def __init__(self, word: str):
        self.word = word
        self.definition_md = ''


class Sutta:
    def __init__(self, uid: str, title: str, content_html: str):
        self.uid = uid
        self.title = title
        self.content_html = content_html


def create_app_dirs():
    if not SIMSAPA_DIR.exists():
        SIMSAPA_DIR.mkdir(parents=True, exist_ok=True)

    if not ASSETS_DIR.exists():
        ASSETS_DIR.mkdir(parents=True, exist_ok=True)



import sys
import os
import logging as _logging
import logging.config
import yaml

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import (QApplication, QSystemTrayIcon, QMenu, QAction)  # type: ignore

from .app.types import AppData, create_app_dirs, APP_DB_PATH  # type: ignore
from .app.windows import AppWindows  # type: ignore
from .layouts.download_appdata import DownloadAppdataWindow

from simsapa.assets import icons_rc  # noqa: F401

logger = _logging.getLogger(__name__)


def main():
    if os.path.exists("logging.yaml"):
        with open("logging.yaml", 'r') as f:
            config = yaml.safe_load(f.read())
            _logging.config.dictConfig(config)

    logger.info("main()")

    create_app_dirs()

    if not APP_DB_PATH.exists():
        dl_app = QApplication(sys.argv)
        w = DownloadAppdataWindow()
        w.show()
        status = dl_app.exec_()
        logger.info(f"Exiting with status {status}.")
        sys.exit(status)

    app = QApplication(sys.argv)

    app_data = AppData()

    # === Create systray ===

    app.setQuitOnLastWindowClosed(False)

    tray = QSystemTrayIcon(QIcon(":bookmark"))
    tray.setVisible(True)

    menu = QMenu()

    ac = QAction(QIcon(":close"), "Quit")
    ac.triggered.connect(app.quit)
    menu.addAction(ac)

    tray.setContextMenu(menu)

    # === Create first window ===

    app_windows = AppWindows(app, app_data)
    app_windows._new_sutta_search_window()

    status = app.exec_()
    logger.info(f"Exiting with status {status}.")
    sys.exit(status)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'simsapa/assets/ui/dictionaries_manager_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DictionariesManagerWindow(object):
    def setupUi(self, DictionariesManagerWindow):
        DictionariesManagerWindow.setObjectName("DictionariesManagerWindow")
        DictionariesManagerWindow.resize(856, 581)
        DictionariesManagerWindow.setBaseSize(QtCore.QSize(800, 600))
        self.central_widget = QtWidgets.QWidget(DictionariesManagerWindow)
        self.central_widget.setObjectName("central_widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.central_widget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.main_layout = QtWidgets.QVBoxLayout()
        self.main_layout.setObjectName("main_layout")
        self.splitter = QtWidgets.QSplitter(self.central_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.dictionary_sources_layout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.dictionary_sources_layout.setContentsMargins(0, 0, 0, 0)
        self.dictionary_sources_layout.setObjectName("dictionary_sources_layout")
        self.dictionary_sources_list = QtWidgets.QListView(self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dictionary_sources_list.setFont(font)
        self.dictionary_sources_list.setObjectName("dictionary_sources_list")
        self.dictionary_sources_layout.addWidget(self.dictionary_sources_list)
        self.verticalLayoutWidget_3 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_3.setObjectName("verticalLayoutWidget_3")
        self.info_layout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_3)
        self.info_layout.setContentsMargins(0, 0, 0, 0)
        self.info_layout.setObjectName("info_layout")
        self.dict_cover = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        self.dict_cover.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dict_cover.setFont(font)
        self.dict_cover.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.dict_cover.setObjectName("dict_cover")
        self.info_layout.addWidget(self.dict_cover)
        self.dict_title = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dict_title.setFont(font)
        self.dict_title.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.dict_title.setObjectName("dict_title")
        self.info_layout.addWidget(self.dict_title)
        self.dict_creator = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dict_creator.setFont(font)
        self.dict_creator.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.dict_creator.setObjectName("dict_creator")
        self.info_layout.addWidget(self.dict_creator)
        self.dict_version = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dict_version.setFont(font)
        self.dict_version.setObjectName("dict_version")
        self.info_layout.addWidget(self.dict_version)
        self.dict_description = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.dict_description.setFont(font)
        self.dict_description.setObjectName("dict_description")
        self.info_layout.addWidget(self.dict_description)
        self.dict_message = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.dict_message.setFont(font)
        self.dict_message.setObjectName("dict_message")
        self.info_layout.addWidget(self.dict_message)
        self.dict_update_button = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dict_update_button.sizePolicy().hasHeightForWidth())
        self.dict_update_button.setSizePolicy(sizePolicy)
        self.dict_update_button.setMinimumSize(QtCore.QSize(100, 40))
        font = QtGui.QFont()
        font.setPointSize(11)
        self.dict_update_button.setFont(font)
        self.dict_update_button.setObjectName("dict_update_button")
        self.info_layout.addWidget(self.dict_update_button)
        self.label = QtWidgets.QLabel(self.verticalLayoutWidget_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setText("")
        self.label.setObjectName("label")
        self.info_layout.addWidget(self.label)
        self.main_layout.addWidget(self.splitter)
        self.horizontalLayout_2.addLayout(self.main_layout)
        DictionariesManagerWindow.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(DictionariesManagerWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 856, 20))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Edit = QtWidgets.QMenu(self.menubar)
        self.menu_Edit.setObjectName("menu_Edit")
        self.menu_Windows = QtWidgets.QMenu(self.menubar)
        self.menu_Windows.setObjectName("menu_Windows")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        self.menu_Library = QtWidgets.QMenu(self.menubar)
        self.menu_Library.setObjectName("menu_Library")
        DictionariesManagerWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(DictionariesManagerWindow)
        self.statusbar.setObjectName("statusbar")
        DictionariesManagerWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(DictionariesManagerWindow)
        self.toolBar.setObjectName("toolBar")
        DictionariesManagerWindow.addToolBar(QtCore.Qt.LeftToolBarArea, self.toolBar)
        self.toolBar_2 = QtWidgets.QToolBar(DictionariesManagerWindow)
        self.toolBar_2.setObjectName("toolBar_2")
        DictionariesManagerWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar_2)
        self.action_Copy = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Copy.setObjectName("action_Copy")
        self.action_Paste = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Paste.setObjectName("action_Paste")
        self.action_Quit = QtWidgets.QAction(DictionariesManagerWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/close"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Quit.setIcon(icon)
        self.action_Quit.setObjectName("action_Quit")
        self.action_Sutta_Search = QtWidgets.QAction(DictionariesManagerWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/book"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Sutta_Search.setIcon(icon1)
        self.action_Sutta_Search.setObjectName("action_Sutta_Search")
        self.action_Dictionary_Search = QtWidgets.QAction(DictionariesManagerWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/dictionary"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Dictionary_Search.setIcon(icon2)
        self.action_Dictionary_Search.setObjectName("action_Dictionary_Search")
        self.action_About = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_About.setObjectName("action_About")
        self.action_Website = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Website.setObjectName("action_Website")
        self.action_Close_Window = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Close_Window.setObjectName("action_Close_Window")
        self.action_Open = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Open.setObjectName("action_Open")
        self.action_Document_Reader = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Document_Reader.setObjectName("action_Document_Reader")
        self.action_Add = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Add.setObjectName("action_Add")
        self.action_Remove = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Remove.setObjectName("action_Remove")
        self.action_Open_Selected = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Open_Selected.setObjectName("action_Open_Selected")
        self.action_Library = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Library.setObjectName("action_Library")
        self.action_Notes = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Notes.setObjectName("action_Notes")
        self.action_Sync_to_Anki = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Sync_to_Anki.setObjectName("action_Sync_to_Anki")
        self.action_Export = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Export.setObjectName("action_Export")
        self.action_Import_from_File = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Import_from_File.setObjectName("action_Import_from_File")
        self.action_Add_from_URL = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Add_from_URL.setObjectName("action_Add_from_URL")
        self.action_Dictionaries_Manager = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Dictionaries_Manager.setObjectName("action_Dictionaries_Manager")
        self.action_Check_Updates = QtWidgets.QAction(DictionariesManagerWindow)
        self.action_Check_Updates.setObjectName("action_Check_Updates")
        self.menu_File.addAction(self.action_Open)
        self.menu_File.addAction(self.action_Close_Window)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.action_Quit)
        self.menu_Edit.addAction(self.action_Copy)
        self.menu_Edit.addAction(self.action_Paste)
        self.menu_Windows.addAction(self.action_Sutta_Search)
        self.menu_Windows.addAction(self.action_Dictionary_Search)
        self.menu_Windows.addAction(self.action_Dictionaries_Manager)
        self.menu_Windows.addAction(self.action_Document_Reader)
        self.menu_Windows.addAction(self.action_Library)
        self.menu_Windows.addAction(self.action_Notes)
        self.menu_Help.addAction(self.action_Website)
        self.menu_Help.addAction(self.action_About)
        self.menu_Library.addAction(self.action_Open_Selected)
        self.menu_Library.addAction(self.action_Add)
        self.menu_Library.addAction(self.action_Remove)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Edit.menuAction())
        self.menubar.addAction(self.menu_Windows.menuAction())
        self.menubar.addAction(self.menu_Library.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.toolBar.addAction(self.action_Sutta_Search)
        self.toolBar.addAction(self.action_Dictionary_Search)
        self.toolBar_2.addAction(self.action_Add_from_URL)
        self.toolBar_2.addAction(self.action_Import_from_File)
        self.toolBar_2.addAction(self.action_Check_Updates)
        self.toolBar_2.addAction(self.action_Sync_to_Anki)
        self.toolBar_2.addAction(self.action_Export)
        self.toolBar_2.addAction(self.action_Remove)

        self.retranslateUi(DictionariesManagerWindow)
        QtCore.QMetaObject.connectSlotsByName(DictionariesManagerWindow)

    def retranslateUi(self, DictionariesManagerWindow):
        _translate = QtCore.QCoreApplication.translate
        DictionariesManagerWindow.setWindowTitle(_translate("DictionariesManagerWindow", "Dictionaries Manager - Simsapa"))
        self.dict_cover.setText(_translate("DictionariesManagerWindow", "Cover"))
        self.dict_title.setText(_translate("DictionariesManagerWindow", "Title"))
        self.dict_creator.setText(_translate("DictionariesManagerWindow", "Creator"))
        self.dict_version.setText(_translate("DictionariesManagerWindow", "Version"))
        self.dict_description.setText(_translate("DictionariesManagerWindow", "Description"))
        self.dict_message.setText(_translate("DictionariesManagerWindow", "Message"))
        self.dict_update_button.setText(_translate("DictionariesManagerWindow", "Update"))
        self.menu_File.setTitle(_translate("DictionariesManagerWindow", "&File"))
        self.menu_Edit.setTitle(_translate("DictionariesManagerWindow", "&Edit"))
        self.menu_Windows.setTitle(_translate("DictionariesManagerWindow", "&Windows"))
        self.menu_Help.setTitle(_translate("DictionariesManagerWindow", "&Help"))
        self.menu_Library.setTitle(_translate("DictionariesManagerWindow", "&Library"))
        self.toolBar.setWindowTitle(_translate("DictionariesManagerWindow", "toolBar"))
        self.toolBar_2.setWindowTitle(_translate("DictionariesManagerWindow", "toolBar_2"))
        self.action_Copy.setText(_translate("DictionariesManagerWindow", "&Copy"))
        self.action_Paste.setText(_translate("DictionariesManagerWindow", "&Paste"))
        self.action_Quit.setText(_translate("DictionariesManagerWindow", "&Quit"))
        self.action_Quit.setShortcut(_translate("DictionariesManagerWindow", "Ctrl+Q"))
        self.action_Sutta_Search.setText(_translate("DictionariesManagerWindow", "&Sutta Search"))
        self.action_Sutta_Search.setShortcut(_translate("DictionariesManagerWindow", "F5"))
        self.action_Dictionary_Search.setText(_translate("DictionariesManagerWindow", "&Dictionary Search"))
        self.action_Dictionary_Search.setShortcut(_translate("DictionariesManagerWindow", "F6"))
        self.action_About.setText(_translate("DictionariesManagerWindow", "&About"))
        self.action_Website.setText(_translate("DictionariesManagerWindow", "&Website"))
        self.action_Close_Window.setText(_translate("DictionariesManagerWindow", "&Close Window"))
        self.action_Open.setText(_translate("DictionariesManagerWindow", "&Open..."))
        self.action_Open.setShortcut(_translate("DictionariesManagerWindow", "Ctrl+O"))
        self.action_Document_Reader.setText(_translate("DictionariesManagerWindow", "D&ocument Reader"))
        self.action_Document_Reader.setToolTip(_translate("DictionariesManagerWindow", "Document Reader"))
        self.action_Document_Reader.setShortcut(_translate("DictionariesManagerWindow", "F7"))
        self.action_Add.setText(_translate("DictionariesManagerWindow", "&Add..."))
        self.action_Remove.setText(_translate("DictionariesManagerWindow", "&Remove"))
        self.action_Remove.setShortcut(_translate("DictionariesManagerWindow", "Del"))
        self.action_Open_Selected.setText(_translate("DictionariesManagerWindow", "&Open Selected"))
        self.action_Library.setText(_translate("DictionariesManagerWindow", "&Library"))
        self.action_Library.setShortcut(_translate("DictionariesManagerWindow", "F8"))
        self.action_Notes.setText(_translate("DictionariesManagerWindow", "&Notes"))
        self.action_Notes.setToolTip(_translate("DictionariesManagerWindow", "Notes"))
        self.action_Notes.setShortcut(_translate("DictionariesManagerWindow", "F9"))
        self.action_Sync_to_Anki.setText(_translate("DictionariesManagerWindow", "&Sync to Anki"))
        self.action_Export.setText(_translate("DictionariesManagerWindow", "&Export..."))
        self.action_Import_from_File.setText(_translate("DictionariesManagerWindow", "&Import from File..."))
        self.action_Add_from_URL.setText(_translate("DictionariesManagerWindow", "&Add from URL..."))
        self.action_Dictionaries_Manager.setText(_translate("DictionariesManagerWindow", "Dictionaries &Manager"))
        self.action_Dictionaries_Manager.setShortcut(_translate("DictionariesManagerWindow", "F10"))
        self.action_Check_Updates.setText(_translate("DictionariesManagerWindow", "&Check Updates"))
from simsapa.assets import icons_rc



from functools import partial
from typing import List
from markdown import markdown
from sqlalchemy.orm import joinedload  # type: ignore

from PyQt5.QtWidgets import (QLabel, QMainWindow)  # type: ignore

from ..app.db_models import DictWord as DbDictWord  # type: ignore
from ..app.types import (AppData, DictWord)  # type: ignore
from ..assets.ui.dictionary_search_window_ui import Ui_DictionarySearchWindow  # type: ignore


class DictionarySearchWindow(QMainWindow, Ui_DictionarySearchWindow):
    def __init__(self, app_data: AppData, parent=None) -> None:
        super().__init__(parent)
        self.setupUi(self)

        self._app_data: AppData = app_data
        self._results: List[DictWord] = []
        self._history: List[DictWord] = []

        self._ui_setup()

        self.statusbar.showMessage("Ready", 3000)

    def _ui_setup(self):
        self.status_msg = QLabel("Word title")
        self.statusbar.addPermanentWidget(self.status_msg)

        self.search_input.setFocus()


class DictionarySearchCtrl:
    def __init__(self, view):
        self._view = view
        self._connect_signals()

    def _handle_query(self):
        query = self._view.search_input.text()
        if len(query) > 3:
            self._view._results = self._word_search_query(query)
            titles = list(map(lambda s: s.word, self._view._results))
            self._view.results_list.clear()
            self._view.results_list.addItems(titles)

    def _set_content_html(self, html):
        self._view.content_html.setText(html)

    def _handle_result_select(self):
        selected_idx = self._view.results_list.currentRow()
        word: DictWord = self._view._results[selected_idx]
        self._show_word(word)

        self._view._history.insert(0, word)
        self._view.history_list.insertItem(0, word.word)

    def _handle_history_select(self):
        selected_idx = self._view.history_list.currentRow()
        word: DictWord = self._view._history[selected_idx]
        self._show_word(word)

    def _show_word(self, word: DictWord):
        self._view.status_msg.setText(word.word)

        def md_to_html(meaning):
            return markdown(meaning.definition_md)

        content_html = "".join(list(map(md_to_html, word.meanings)))

        html = """
<!doctype html>
<html>
  <head>
    <meta charset="utf-8">
    <style>%s</style>
  </head>
  <body>
  %s
  </body>
</html>
""" % ('', content_html)

        self._set_content_html(html)

    def _word_search_query(self, query: str):
        results = self._view._app_data.app_db_session \
                                      .query(DbDictWord) \
                                      .options(joinedload(DbDictWord.meanings)) \
                                      .filter(DbDictWord.word.like(f"%{query}%")) \
                                      .all()
        return results

    def _connect_signals(self):
        self._view.action_Close_Window \
            .triggered.connect(partial(self._view.close))

        self._view.search_button.clicked.connect(partial(self._handle_query))
        self._view.search_input.textChanged.connect(partial(self._handle_query))
        # self._view.search_input.returnPressed.connect(partial(self._update_result))
        self._view.results_list.itemSelectionChanged.connect(partial(self._handle_result_select))
        self._view.history_list.itemSelectionChanged.connect(partial(self._handle_history_select))



import os
import shutil
import json
import semver
import logging as _logging
from functools import partial
from typing import List, Optional
from pathlib import Path
from zipfile import ZipFile
# import pandas
from sqlalchemy.sql import func  # type: ignore

from PyQt5.QtCore import QAbstractListModel, Qt  # type: ignore
from PyQt5.QtGui import QFont, QIcon  # type: ignore
from PyQt5.QtWidgets import (QLabel, QMainWindow,  # type: ignore
                             QMessageBox, QInputDialog, QLineEdit)
from simsapa.assets import icons_rc  # noqa: F401

from ..app.db_models import DictionarySource  # type: ignore
from ..app.types import AppData, ASSETS_DIR  # type: ignore
from ..app.helpers import download_file  # type: ignore
from ..assets.ui.dictionaries_manager_window_ui import Ui_DictionariesManagerWindow  # type: ignore

logger = _logging.getLogger(__name__)


class DictionarySourceListModel(QAbstractListModel):
    def __init__(self, *args, dictionaries=None, **kwargs):
        super(DictionarySourceListModel, self).__init__(*args, **kwargs)
        self.dictionaries = dictionaries or []

    def data(self, index, role):
        if role == Qt.DisplayRole:
            return self.dictionaries[index.row()].title

        if role == Qt.DecorationRole:
            if self.dictionaries[index.row()].has_update:
                return QIcon(":sync")

    def rowCount(self, index):
        return len(self.dictionaries)


class DictionariesManagerWindow(QMainWindow, Ui_DictionariesManagerWindow):
    def __init__(self, app_data: AppData, parent=None) -> None:
        super().__init__(parent)
        self.setupUi(self)

        self._app_data: AppData = app_data

        self.model = DictionarySourceListModel(dictionaries=self._get_all_dictionary_sources())

        self.dictionary_sources_list.setModel(self.model)
        self.sel_model = self.dictionary_sources_list.selectionModel()

        self._ui_setup()
        self._connect_signals()

        self.statusbar.showMessage("Ready", 3000)

    def _ui_setup(self):
        self.status_msg = QLabel("")
        self.statusbar.addPermanentWidget(self.status_msg)

        self._show_dictionary_source_clear()

    def _get_all_dictionary_sources(self) -> List[DictionarySource]:
        return self._app_data.user_db_session.query(DictionarySource).all()

    def get_selected_dictionary_source(self) -> Optional[DictionarySource]:
        a = self.dictionary_sources_list.selectedIndexes()
        if not a:
            return None

        item = a[0]
        return self.model.dictionaries[item.row()]

    def remove_selected_dictionary_source(self):
        a = self.dictionary_sources_list.selectedIndexes()
        if not a:
            return None

        # Remove from model
        item = a[0]
        dict_id = self.model.dictionaries[item.row()].id

        del self.model.dictionaries[item.row()]
        self.model.layoutChanged.emit()
        self.dictionary_sources_list.clearSelection()
        self._show_dictionary_source_clear()

        # Remove from database

        db_item = self._app_data.user_db_session \
                                .query(DictionarySource) \
                                .filter(DictionarySource.id == dict_id) \
                                .first()
        self._app_data.user_db_session.delete(db_item)
        self._app_data.user_db_session.commit()

    def _handle_dictionary_source_select(self):
        doc = self.get_selected_dictionary_source()
        if doc:
            self._show_dictionary_source(doc)
        else:
            self._show_dictionary_source_clear()

    def _show_dictionary_source_clear(self):
        self.status_msg.clear()
        self.dict_creator.clear()
        self.dict_cover.clear()
        self.dict_message.clear()
        self.dict_title.clear()
        self.dict_version.clear()
        self.dict_description.clear()
        self.dict_update_button.hide()

    def _show_dictionary_source(self, dict_source: DictionarySource):
        self.status_msg.setText(dict_source.title)
        self.dict_creator.setText(dict_source.creator)
        self.dict_title.setText(dict_source.title)
        self.dict_version.setText(dict_source.version)
        self.dict_description.setText(dict_source.description)

        if dict_source.has_update:
            self.dict_message.setText("An updated version is available.")
            font = QFont()
            font.setItalic(True)
            self.dict_message.setFont(font)
            self.dict_update_button.show()
        else:
            self.dict_message.clear()
            self.dict_update_button.hide()

    def add_from_url(self, json_url):
        temp_dir: Path = ASSETS_DIR.joinpath('temp')
        if not temp_dir.exists():
            os.mkdir(temp_dir)

        json_path = download_file(json_url, temp_dir)
        with open(json_path, 'r') as f:
            remote_info = json.loads(f.read())

        zip_path = download_file(remote_info['data_zip_url'], temp_dir)

        with ZipFile(zip_path, 'r') as z:
            z.extractall(temp_dir)

        # xlsx_path = temp_dir.joinpath('data.xlsx')

        # xlsx_data_df = pandas.read_excel(xlsx_path, sheet_name='Words')
        # xlsx_json = xlsx_data_df.to_json()

        # TODO: add words to database

        # Remove downloaded files
        shutil.rmtree(temp_dir)

        # Add DictionarySource

        item = self._app_data.user_db_session \
                             .query(DictionarySource) \
                             .filter(DictionarySource.info_json_url == json_url) \
                             .first()

        if item is None:
            logger.info(f"Add new: {remote_info['title']}")
            db_source = DictionarySource(
                title=remote_info['title'],
                creator=remote_info['creator'],
                description=remote_info['description'],
                contact_email=remote_info['contact_email'],
                version=remote_info['version'],
                has_update=False,
                info_json_url=json_url,
                data_zip_url=remote_info['data_zip_url'],
                created_at=func.now(),
            )
            try:
                self._app_data.user_db_session.add(db_source)
                self._app_data.user_db_session.commit()

                # Add to model
                self.model.dictionaries.append(db_source)

            except Exception as e:
                logger.error(e)

        else:
            logger.info(f"Update: {item.title}")

            values = {
                'title': remote_info['title'],
                'creator': remote_info['creator'],
                'description': remote_info['description'],
                'contact_email': remote_info['contact_email'],
                'version': remote_info['version'],
                'has_update': False,
                'info_json_url': json_url,
                'data_zip_url': remote_info['data_zip_url'],
                'updated_at': func.now()
            }
            try:
                self._app_data.user_db_session \
                              .query(DictionarySource) \
                              .filter(DictionarySource.id == item.id) \
                              .update(values)

                self._app_data.user_db_session.commit()

            except Exception as e:
                logger.error(e)

        self._handle_dictionary_source_select()
        self.model.layoutChanged.emit()

    def _add_from_url_dialog(self):
        url, ok = QInputDialog.getText(self,
                                       "Add from URL...",
                                       "URL of JSON info of the dictionary:",
                                       QLineEdit.Normal,
                                       "")
        if ok:
            self.add_from_url(url)

    def _remove_dictionary_source_dialog(self):
        dict_source = self.get_selected_dictionary_source()
        if not dict_source:
            return

        reply = QMessageBox.question(self,
                                     'Remove Dictionary Source...',
                                     'Remove this Dictionary source?',
                                     QMessageBox.Yes | QMessageBox.No,
                                     QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.remove_selected_dictionary_source()

    def _check_updates(self):
        temp_dir: Path = ASSETS_DIR.joinpath('temp')
        if not temp_dir.exists():
            os.mkdir(temp_dir)

        for d in self.model.dictionaries:
            json_path = download_file(d.info_json_url, temp_dir)
            with open(json_path, 'r') as f:
                remote_info = json.loads(f.read())
                if semver.compare(remote_info['version'], d.version) == 1:

                    d.has_update = True
                    self._app_data.user_db_session.commit()

        self.model.layoutChanged.emit()

    def _update_selected_dictionary_source(self):
        dict_source = self.get_selected_dictionary_source()
        if dict_source:
            self.add_from_url(dict_source.info_json_url)

    def _connect_signals(self):
        self.action_Close_Window \
            .triggered.connect(partial(self.close))

        self.action_Add_from_URL \
            .triggered.connect(partial(self._add_from_url_dialog))

        self.action_Check_Updates \
            .triggered.connect(partial(self._check_updates))

        self.action_Remove \
            .triggered.connect(partial(self._remove_dictionary_source_dialog))

        self.sel_model.selectionChanged.connect(partial(self._handle_dictionary_source_select))

        self.dict_update_button.clicked.connect(partial(self._update_selected_dictionary_source))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'simsapa/assets/ui/notes_browser_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NotesBrowserWindow(object):
    def setupUi(self, NotesBrowserWindow):
        NotesBrowserWindow.setObjectName("NotesBrowserWindow")
        NotesBrowserWindow.resize(856, 581)
        NotesBrowserWindow.setBaseSize(QtCore.QSize(800, 600))
        self.central_widget = QtWidgets.QWidget(NotesBrowserWindow)
        self.central_widget.setObjectName("central_widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.central_widget)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.main_layout = QtWidgets.QVBoxLayout()
        self.main_layout.setObjectName("main_layout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.central_widget)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.front_input = QtWidgets.QPlainTextEdit(self.central_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.front_input.sizePolicy().hasHeightForWidth())
        self.front_input.setSizePolicy(sizePolicy)
        self.front_input.setMinimumSize(QtCore.QSize(0, 50))
        self.front_input.setMaximumSize(QtCore.QSize(16777215, 50))
        self.front_input.setObjectName("front_input")
        self.verticalLayout.addWidget(self.front_input)
        self.label_2 = QtWidgets.QLabel(self.central_widget)
        self.label_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.back_input = QtWidgets.QPlainTextEdit(self.central_widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.back_input.sizePolicy().hasHeightForWidth())
        self.back_input.setSizePolicy(sizePolicy)
        self.back_input.setMinimumSize(QtCore.QSize(0, 50))
        self.back_input.setMaximumSize(QtCore.QSize(16777215, 50))
        self.back_input.setObjectName("back_input")
        self.verticalLayout.addWidget(self.back_input)
        self.main_layout.addLayout(self.verticalLayout)
        self.searchbar_layout = QtWidgets.QHBoxLayout()
        self.searchbar_layout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.searchbar_layout.setObjectName("searchbar_layout")
        self.search_input = QtWidgets.QLineEdit(self.central_widget)
        self.search_input.setMinimumSize(QtCore.QSize(0, 35))
        self.search_input.setClearButtonEnabled(True)
        self.search_input.setObjectName("search_input")
        self.searchbar_layout.addWidget(self.search_input)
        self.search_button = QtWidgets.QPushButton(self.central_widget)
        self.search_button.setMinimumSize(QtCore.QSize(0, 40))
        self.search_button.setObjectName("search_button")
        self.searchbar_layout.addWidget(self.search_button)
        self.main_layout.addLayout(self.searchbar_layout)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.notes_list = QtWidgets.QListView(self.central_widget)
        self.notes_list.setObjectName("notes_list")
        self.verticalLayout_2.addWidget(self.notes_list)
        self.main_layout.addLayout(self.verticalLayout_2)
        self.horizontalLayout_2.addLayout(self.main_layout)
        NotesBrowserWindow.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(NotesBrowserWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 856, 20))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Edit = QtWidgets.QMenu(self.menubar)
        self.menu_Edit.setObjectName("menu_Edit")
        self.menu_Windows = QtWidgets.QMenu(self.menubar)
        self.menu_Windows.setObjectName("menu_Windows")
        self.menu_Help = QtWidgets.QMenu(self.menubar)
        self.menu_Help.setObjectName("menu_Help")
        self.menu_Notes = QtWidgets.QMenu(self.menubar)
        self.menu_Notes.setObjectName("menu_Notes")
        NotesBrowserWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(NotesBrowserWindow)
        self.statusbar.setObjectName("statusbar")
        NotesBrowserWindow.setStatusBar(self.statusbar)
        self.toolBar_2 = QtWidgets.QToolBar(NotesBrowserWindow)
        self.toolBar_2.setObjectName("toolBar_2")
        NotesBrowserWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar_2)
        self.action_Copy = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Copy.setObjectName("action_Copy")
        self.action_Paste = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Paste.setObjectName("action_Paste")
        self.action_Quit = QtWidgets.QAction(NotesBrowserWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/close"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Quit.setIcon(icon)
        self.action_Quit.setObjectName("action_Quit")
        self.action_Sutta_Search = QtWidgets.QAction(NotesBrowserWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/book"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Sutta_Search.setIcon(icon1)
        self.action_Sutta_Search.setObjectName("action_Sutta_Search")
        self.action_Dictionary_Search = QtWidgets.QAction(NotesBrowserWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/dictionary"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_Dictionary_Search.setIcon(icon2)
        self.action_Dictionary_Search.setObjectName("action_Dictionary_Search")
        self.action_About = QtWidgets.QAction(NotesBrowserWindow)
        self.action_About.setObjectName("action_About")
        self.action_Website = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Website.setObjectName("action_Website")
        self.action_Close_Window = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Close_Window.setObjectName("action_Close_Window")
        self.action_Open = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Open.setObjectName("action_Open")
        self.action_Document_Reader = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Document_Reader.setObjectName("action_Document_Reader")
        self.action_Add = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Add.setObjectName("action_Add")
        self.action_Remove = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Remove.setObjectName("action_Remove")
        self.action_Open_Selected = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Open_Selected.setObjectName("action_Open_Selected")
        self.action_Library = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Library.setObjectName("action_Library")
        self.action_Sync_to_Anki = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Sync_to_Anki.setObjectName("action_Sync_to_Anki")
        self.action_Notes = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Notes.setObjectName("action_Notes")
        self.action_Dictionaries_Manager = QtWidgets.QAction(NotesBrowserWindow)
        self.action_Dictionaries_Manager.setObjectName("action_Dictionaries_Manager")
        self.menu_File.addAction(self.action_Open)
        self.menu_File.addAction(self.action_Close_Window)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.action_Quit)
        self.menu_Edit.addAction(self.action_Copy)
        self.menu_Edit.addAction(self.action_Paste)
        self.menu_Windows.addAction(self.action_Sutta_Search)
        self.menu_Windows.addAction(self.action_Dictionary_Search)
        self.menu_Windows.addAction(self.action_Dictionaries_Manager)
        self.menu_Windows.addAction(self.action_Document_Reader)
        self.menu_Windows.addAction(self.action_Library)
        self.menu_Windows.addAction(self.action_Notes)
        self.menu_Help.addAction(self.action_Website)
        self.menu_Help.addAction(self.action_About)
        self.menu_Notes.addAction(self.action_Sync_to_Anki)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Edit.menuAction())
        self.menubar.addAction(self.menu_Windows.menuAction())
        self.menubar.addAction(self.menu_Notes.menuAction())
        self.menubar.addAction(self.menu_Help.menuAction())
        self.toolBar_2.addAction(self.action_Add)
        self.toolBar_2.addAction(self.action_Remove)
        self.toolBar_2.addAction(self.action_Sync_to_Anki)

        self.retranslateUi(NotesBrowserWindow)
        QtCore.QMetaObject.connectSlotsByName(NotesBrowserWindow)

    def retranslateUi(self, NotesBrowserWindow):
        _translate = QtCore.QCoreApplication.translate
        NotesBrowserWindow.setWindowTitle(_translate("NotesBrowserWindow", "Notes - Simsapa"))
        self.label.setText(_translate("NotesBrowserWindow", "Front"))
        self.label_2.setText(_translate("NotesBrowserWindow", "Back"))
        self.search_button.setText(_translate("NotesBrowserWindow", "Search"))
        self.menu_File.setTitle(_translate("NotesBrowserWindow", "&File"))
        self.menu_Edit.setTitle(_translate("NotesBrowserWindow", "&Edit"))
        self.menu_Windows.setTitle(_translate("NotesBrowserWindow", "&Windows"))
        self.menu_Help.setTitle(_translate("NotesBrowserWindow", "&Help"))
        self.menu_Notes.setTitle(_translate("NotesBrowserWindow", "&Notes"))
        self.toolBar_2.setWindowTitle(_translate("NotesBrowserWindow", "toolBar_2"))
        self.action_Copy.setText(_translate("NotesBrowserWindow", "&Copy"))
        self.action_Paste.setText(_translate("NotesBrowserWindow", "&Paste"))
        self.action_Quit.setText(_translate("NotesBrowserWindow", "&Quit"))
        self.action_Quit.setShortcut(_translate("NotesBrowserWindow", "Ctrl+Q"))
        self.action_Sutta_Search.setText(_translate("NotesBrowserWindow", "&Sutta Search"))
        self.action_Sutta_Search.setShortcut(_translate("NotesBrowserWindow", "F5"))
        self.action_Dictionary_Search.setText(_translate("NotesBrowserWindow", "&Dictionary Search"))
        self.action_Dictionary_Search.setShortcut(_translate("NotesBrowserWindow", "F6"))
        self.action_About.setText(_translate("NotesBrowserWindow", "&About"))
        self.action_Website.setText(_translate("NotesBrowserWindow", "&Website"))
        self.action_Close_Window.setText(_translate("NotesBrowserWindow", "&Close Window"))
        self.action_Open.setText(_translate("NotesBrowserWindow", "&Open..."))
        self.action_Open.setShortcut(_translate("NotesBrowserWindow", "Ctrl+O"))
        self.action_Document_Reader.setText(_translate("NotesBrowserWindow", "D&ocument Reader"))
        self.action_Document_Reader.setToolTip(_translate("NotesBrowserWindow", "Document Reader"))
        self.action_Document_Reader.setShortcut(_translate("NotesBrowserWindow", "F7"))
        self.action_Add.setText(_translate("NotesBrowserWindow", "&Add..."))
        self.action_Remove.setText(_translate("NotesBrowserWindow", "&Remove"))
        self.action_Remove.setShortcut(_translate("NotesBrowserWindow", "Del"))
        self.action_Open_Selected.setText(_translate("NotesBrowserWindow", "&Open Selected"))
        self.action_Library.setText(_translate("NotesBrowserWindow", "&Library"))
        self.action_Library.setShortcut(_translate("NotesBrowserWindow", "F8"))
        self.action_Sync_to_Anki.setText(_translate("NotesBrowserWindow", "&Sync to Anki"))
        self.action_Notes.setText(_translate("NotesBrowserWindow", "&Notes"))
        self.action_Notes.setToolTip(_translate("NotesBrowserWindow", "Notes"))
        self.action_Notes.setShortcut(_translate("NotesBrowserWindow", "F9"))
        self.action_Dictionaries_Manager.setText(_translate("NotesBrowserWindow", "Dictionaries &Manager"))
        self.action_Dictionaries_Manager.setShortcut(_translate("NotesBrowserWindow", "F10"))
from simsapa.assets import icons_rc



import logging as _logging
import requests
from functools import partial
from typing import List, Optional

from PyQt5.QtCore import QAbstractListModel, Qt
from PyQt5.QtWidgets import (QLabel, QMainWindow,  QMessageBox)  # type: ignore
from sqlalchemy.sql import func  # type: ignore

from simsapa.assets import icons_rc  # noqa: F401

from ..app.db_models import Note  # type: ignore
from ..app.types import AppData  # type: ignore
from ..assets.ui.notes_browser_window_ui import Ui_NotesBrowserWindow  # type: ignore

logger = _logging.getLogger(__name__)


class NoteListModel(QAbstractListModel):
    def __init__(self, *args, notes=None, **kwargs):
        super(NoteListModel, self).__init__(*args, **kwargs)
        self.notes = notes or []

    def data(self, index, role):
        if role == Qt.DisplayRole:
            text = self.notes[index.row()].front + " " + self.notes[index.row()].back
            return text

    def rowCount(self, index):
        return len(self.notes)


class NotesBrowserWindow(QMainWindow, Ui_NotesBrowserWindow):
    def __init__(self, app_data: AppData, parent=None) -> None:
        super().__init__(parent)
        self.setupUi(self)

        self._app_data: AppData = app_data

        self.model = NoteListModel(notes=self._get_all_notes())

        self.notes_list.setModel(self.model)
        self.sel_model = self.notes_list.selectionModel()

        self._ui_setup()
        self._connect_signals()

        self.statusbar.showMessage("Ready", 3000)

    def _ui_setup(self):
        self.status_msg = QLabel("")
        self.statusbar.addPermanentWidget(self.status_msg)

        self.front_input.setFocus()
        self._show_note_clear()

    def _get_all_notes(self) -> List[Note]:
        return self._app_data.user_db_session.query(Note).all()

    def _handle_query(self):
        pass

    def get_selected_note(self) -> Optional[Note]:
        a = self.notes_list.selectedIndexes()
        if not a:
            return None

        item = a[0]
        return self.model.notes[item.row()]

    def remove_selected_note(self):
        a = self.notes_list.selectedIndexes()
        if not a:
            return None

        # Remove from model
        item = a[0]
        note_id = self.model.notes[item.row()].id

        del self.model.notes[item.row()]
        self.model.layoutChanged.emit()
        self.notes_list.clearSelection()
        self._show_note_clear()

        # Remove from database

        db_item = self._app_data.user_db_session \
                                .query(Note) \
                                .filter(Note.id == note_id) \
                                .first()
        self._app_data.user_db_session.delete(db_item)
        self._app_data.user_db_session.commit()

    def _handle_note_select(self):
        note = self.get_selected_note()
        if note:
            self._show_note(note)

    def _show_note_clear(self):
        self.status_msg.clear()
        self.front_input.clear()
        self.back_input.clear()

    def _show_note(self, note: Note):
        self.front_input.setPlainText(note.front)
        self.back_input.setPlainText(note.back)

    def _notes_search_query(self, query: str):
        return []

    def add_note(self):

        front = self.front_input.toPlainText()
        back = self.back_input.toPlainText()

        if len(front) == 0 and len(back) == 0:
            logger.info("Empty content, cancel adding.")
            return

        # Insert database record

        logger.info("Adding new note")

        db_note = Note(
            front=front,
            back=back,
            created_at=func.now(),
        )

        try:
            self._app_data.user_db_session.add(db_note)
            self._app_data.user_db_session.commit()

            # Add to model
            self.model.notes.append(db_note)

        except Exception as e:
            logger.error(e)

        self.model.layoutChanged.emit()

    def update_selected_note_front(self):
        note = self.get_selected_note()
        if note is None:
            return

        note.front = self.front_input.toPlainText()

        self._app_data.user_db_session.commit()
        self.model.layoutChanged.emit()

    def update_selected_note_back(self):
        note = self.get_selected_note()
        if note is None:
            return

        note.back = self.back_input.toPlainText()

        self._app_data.user_db_session.commit()
        self.model.layoutChanged.emit()

    def remove_note_dialog(self):
        note = self.get_selected_note()
        if not note:
            return

        reply = QMessageBox.question(self,
                                     'Remove Note...',
                                     'Remove this item?',
                                     QMessageBox.Yes | QMessageBox.No,
                                     QMessageBox.No)
        if reply == QMessageBox.Yes:
            self.remove_selected_note()

    def test_anki_live_or_notify(self) -> bool:
        if not is_anki_live():
            QMessageBox.information(self,
                                    "Anki is not connected",
                                    "Anki must be running with the AnkiConnect plugin.",
                                    QMessageBox.Ok)
            return False
        return True

    def sync_to_anki(self):
        if not self.test_anki_live_or_notify():
            return

        res = anki_invoke('deckNames')
        if 'Simsapa' not in res:
            anki_invoke('createDeck', deck='Simsapa')

        for note in self.model.notes:

            if note.anki_note_id:
                logger.info("Updating Anki note...")

                anki_invoke(
                    'updateNoteFields',
                    note={
                        'id': note.anki_note_id,
                        'fields': {
                            'Front': note.front,
                            'Back': note.back,
                        },
                    }
                )

            else:
                logger.info("Creating Anki note...")

                res = anki_invoke(
                    'addNote',
                    note={
                        'deckName': 'Simsapa',
                        'modelName': 'Basic',
                        'fields': {
                            'Front': note.front,
                            'Back': note.back,
                        },
                        'options': {
                            'allowDuplicate': True,
                            'duplicateScope': 'deck',
                            'duplicateScopeOptions': {
                                'deckName': 'Simsapa',
                                'checkChildren': False
                            }
                        },
                    }
                )

                note.anki_note_id = res
                self._app_data.user_db_session.commit()

    def _connect_signals(self):
        self.action_Close_Window \
            .triggered.connect(partial(self.close))

        self.action_Add \
            .triggered.connect(partial(self.add_note))

        self.action_Remove \
            .triggered.connect(partial(self.remove_note_dialog))

        self.action_Sync_to_Anki \
            .triggered.connect(partial(self.sync_to_anki))

        self.search_button.clicked.connect(partial(self._handle_query))
        self.search_input.textChanged.connect(partial(self._handle_query))

        self.front_input.textChanged.connect(partial(self.update_selected_note_front))
        self.back_input.textChanged.connect(partial(self.update_selected_note_back))

        self.sel_model.selectionChanged.connect(partial(self._handle_note_select))


def is_anki_live() -> bool:
    try:
        res = requests.get('http://localhost:8765')
    except Exception:
        return False

    return res.status_code == 200


def anki_request(action, **params):
    return {'action': action, 'params': params, 'version': 6}


def anki_invoke(action, **params):
    res = requests.get(url='http://localhost:8765', json=anki_request(action, **params))
    response = res.json()

    if len(response) != 2:
        raise Exception('response has an unexpected number of fields')

    if 'error' not in response:
        raise Exception('response is missing required error field')

    if 'result' not in response:
        raise Exception('response is missing required result field')

    if response['error'] is not None:
        raise Exception(response['error'])

    return response['result']



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simsapa',
 'simsapa.app',
 'simsapa.assets',
 'simsapa.assets.ui',
 'simsapa.layouts']

package_data = \
{'': ['*'],
 'simsapa.assets': ['icons/*', 'icons/32x32/*', 'icons/gif/*', 'icons/svg/*']}

install_requires = \
['Markdown>=3.3.4,<4.0.0',
 'PyPDF2>=1.26.0,<2.0.0',
 'PyQt5==5.15.0',
 'PyYAML>=5.4.1,<6.0.0',
 'SQLAlchemy-Utils>=0.37.2,<0.38.0',
 'SQLAlchemy>=1.4.6,<2.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'epub_meta>=0.0.7,<0.0.8',
 'openpyxl>=3.0.7,<4.0.0',
 'pandas>=1.2.4,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'semver>=2.13.0,<3.0.0']

extras_require = \
{':sys_platform == "darwin"': ['PyMuPDF>=1.18.13,<2.0.0'],
 ':sys_platform == "linux" and platform_machine == "x86_64"': ['PyMuPDF>=1.18.13,<2.0.0'],
 ':sys_platform == "win32"': ['PyMuPDF>=1.18.13,<2.0.0']}

setup_kwargs = {
    'name': 'simsapa',
    'version': '0.1.6',
    'description': 'Simsapa Dhamma Reader',
    'long_description': None,
    'author': 'Gambhiro',
    'author_email': 'gambhiro.bhikkhu.85@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
